package dev.micle.totemofreviving.network;

import dev.micle.totemofreviving.TotemOfReviving;
import dev.micle.totemofreviving.items.StrawTotemItem;
import dev.micle.totemofreviving.items.TotemOfRevivingItem;
import dev.micle.totemofreviving.utils.Utils;
import net.minecraft.ChatFormatting;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.stats.Stats;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.effect.MobEffectInstance;
import net.minecraft.world.effect.MobEffects;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.GameType;
import net.minecraftforge.fmllegacy.network.NetworkEvent;

import java.util.UUID;
import java.util.function.Supplier;

public class C2SRequestPlayerRevive {
    private final UUID player_uuid;
    private final InteractionHand hand;

    public C2SRequestPlayerRevive(final UUID player_uuid, final InteractionHand hand) {
        this.player_uuid = player_uuid;
        this.hand = hand;
    }

    public static void encode(final C2SRequestPlayerRevive msg, FriendlyByteBuf packet_buffer) {
        packet_buffer.writeUUID( msg.player_uuid);
        packet_buffer.writeEnum(msg.hand);
    }

    public static C2SRequestPlayerRevive decode(final FriendlyByteBuf packet_buffer) {
        return new C2SRequestPlayerRevive(packet_buffer.readUUID(), packet_buffer.readEnum(InteractionHand.class));
    }

    public static void handle(final C2SRequestPlayerRevive msg, final Supplier<NetworkEvent.Context> context_supplier) {
        final NetworkEvent.Context context = context_supplier.get();
        context.enqueueWork(() -> {
            final ServerPlayer sender = TotemOfReviving.players.getPlayer(msg.player_uuid);
            if (sender == null) { return; }

            ItemStack item = sender.getItemInHand(msg.hand);
            if (item.getOrCreateTag().getInt(TotemOfRevivingItem.TAG_TARGET_INDEX) > TotemOfReviving.players.getPlayerCount()-1 || item.getOrCreateTag().getString(TotemOfRevivingItem.TAG_TARGET_NAME).equals("")) {
                sender.sendMessage(new TextComponent(ChatFormatting.RED + "Error getting target! (Try selecting the target again)"), sender.getUUID());
            } else {
                ServerPlayer player_to_revive = TotemOfReviving.players.getPlayerByName(item.getOrCreateTag().getString(TotemOfRevivingItem.TAG_TARGET_NAME));
                ServerLevel player_to_revive_world = player_to_revive.getLevel();
                ServerLevel sender_world = sender.getLevel();
                int required_charge = player_to_revive.getStats().getValue(Stats.CUSTOM.get(Stats.DEATHS));
                if (player_to_revive.isSpectator()) {
                    if (player_to_revive_world.equals(sender_world)) {
                        if (item.getOrCreateTag().getInt(TotemOfRevivingItem.TAG_CHARGE_AMOUNT) >= required_charge) {
                            if (item.getOrCreateTag().contains(StrawTotemItem.TAG_FAIL_CHANCE)) {
                                int fail_chance = item.getOrCreateTag().getInt(StrawTotemItem.TAG_FAIL_CHANCE);
                                if (Utils.randomIntRange(0, 100) <= fail_chance) {
                                    item.getOrCreateTag().putInt(StrawTotemItem.TAG_CHARGE_AMOUNT, item.getOrCreateTag().getInt(StrawTotemItem.TAG_CHARGE_AMOUNT)-required_charge);
                                    item.getOrCreateTag().putInt(StrawTotemItem.TAG_FAIL_CHANCE, fail_chance-(5*required_charge));
                                    sender.addEffect(new MobEffectInstance(MobEffects.POISON, ((fail_chance*10) / 2), 1));
                                    return;
                                } else {
                                    item.getOrCreateTag().putInt(StrawTotemItem.TAG_FAIL_CHANCE, fail_chance-(5*required_charge));
                                }
                            }
                            player_to_revive.teleportTo(sender.getX(), sender.getY(), sender.getZ());
                            player_to_revive.setGameMode(GameType.SURVIVAL);
                            item.getOrCreateTag().putInt(TotemOfRevivingItem.TAG_CHARGE_AMOUNT, item.getOrCreateTag().getInt(TotemOfRevivingItem.TAG_CHARGE_AMOUNT) - required_charge);
                            sender.sendMessage(new TextComponent(ChatFormatting.GRAY + "Successfully revived " + ChatFormatting.DARK_GRAY + player_to_revive.getDisplayName().getString()), sender.getUUID());
                        } else {
                            sender.sendMessage(new TextComponent(ChatFormatting.GRAY + "Not enough charge! Required charge is: " + ChatFormatting.DARK_GRAY + required_charge), sender.getUUID());
                        }
                    } else {
                        sender.sendMessage(new TextComponent(ChatFormatting.DARK_GRAY + player_to_revive.getDisplayName().getString() + ChatFormatting.GRAY + " is not in this dimension!"), sender.getUUID());
                    }
                } else {
                    sender.sendMessage(new TextComponent(ChatFormatting.DARK_GRAY + player_to_revive.getDisplayName().getString() + ChatFormatting.GRAY + " is not dead!"), sender.getUUID());
                }
            }
        });
        context.setPacketHandled(true);
    }
}
